\name{hdpmn}
\alias{hdpmn}
\title{MCMC for the hierarchical DP mixture}
\description{
  Initializes and runs posterior MCMC for the
  hierarchical DP mixture of normals for dependent random probability
  measures. 
}
\usage{
hdpmn(Z = NULL, study = NULL,
      npa = NULL,  nstudies = NULL,
      n.iter = 1000, n.discard = 100, n.batch = 50, 
      verbose = 3, seed1 = 981963, seed2 = 6869504,
      mcmc.eps = 0, eps = 0.1, ae = 1, be = 1, pe1 = 0.1, pe0 = 0.1,
      pz = NULL, px = NULL,
      m.prior = 0, B.prior = 0, S.prior = 0,
      alpha.prior = 0, n.predupdate = 100,
      S.init = NULL, q = 5, R = NULL,
      B.init = NULL,  cc = 5, C = NULL,
      m.init = NULL, a = NULL,  A = NULL,
      alpha = 1, a0 = 1, b0 = 1,
      k0 = NULL, header = T)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{Z}{Data and covariates,
    as a (\code{n} by \code{p}) matrix or file name, where
    \eqn{p=\code{pz}+\code{px}}.
    The i-th row of \code{Z} reports the i-th observation,
    as a combined vector of the \code{pz}-dimensional response
    and a \code{px}-dimensional covariate vector.}
  \item{study}{(1 by \code{n}) vector of study indicators.
    The i-th index is the study j that response i belongs to.}
  \item{npa}{Total number of observations (patients),
    counting across all studies.}
  \item{nstudies}{Number of studies, \eqn{=J} in \eqn{j=1,\ldots,J}.}
  \item{n.iter}{Number of Markov chain Monte Carlo iterations.}
  \item{n.discard}{Initial burn-in to be discarded.}
  \item{n.batch}{Save imputed paramter values every \code{n.batch}
    iterations.}
  \item{verbose}{0 is silent, 3 is verbose.}
  \item{seed1}{Random variate seed.}
  \item{seed2}{Random variate seed.}
  \item{mcmc.eps}{Indicator for resampling \eqn{\epsilon}. If zero,
    \eqn{\epsilon} will be fixed.}
  \item{eps}{Initial value for \eqn{\epsilon}.}
  \item{ae}{Prior paramters for a Beta prior on \eqn{\epsilon}}
  \item{be}{}
  \item{pe1}{Point mass at \eqn{\epsilon=1}}
  \item{pe0}{Point mass at \eqn{\epsilon=0}}
  \item{pz}{Dimension of the data vector \eqn{\theta_{ji}},
    without the covariate vector.}
  \item{px}{Dimension of a covariate vector.}
  \item{m.prior}{Indicator for resampling \eqn{m}, the mean
    of the normal base measure.}
  \item{B.prior}{Indicator for resampling \eqn{B}, the covariance matrix
    of the normal base measure.}
  \item{S.prior}{Indicator for resampling \eqn{S}, the covariance matrix
    of the normal kernel in the DP mixture.}
  \item{alpha.prior}{Indicator for resampling \eqn{\alpha}, the
    total mass parameter of the DP prior.}
  \item{n.predupdate}{Batch size to update posterior predictive
    inference in the MCMC simulation.}
  \item{S.init}{initial value for \eqn{S}. \code{NULL} is allowed.}
  \item{q}{degrees of freedom in the inverse Wishart prior for \eqn{S}}
  \item{R}{matrix-variate parameter for the inverse Wishart prior for
    \eqn{S}} 
  \item{B.init}{initial value for \eqn{B}, the covariance matrix
    in the DP base measure.}
  \item{cc}{degrees of freedom in the inverse Wishart prior for \eqn{B}}
  \item{C}{matrix-variate parameter for the inverse Wishart prior for
    \eqn{B}}
  \item{m.init}{initial value for \eqn{m}.}
  \item{a}{mean of the normal hyperprior for \eqn{m}.}
  \item{A}{covariance matrix of the normal hyperprior for \eqn{m}.}
  \item{alpha}{initial value of the total mass parameter in the DP
    prior.} 
  \item{a0}{hyperparameters in the Gamma prior for \eqn{alpha}.}
  \item{b0}{}
  \item{k0}{initial number of clusters.}
  \item{work.dir}{directory where working files will be saved.
    \code{NULL} indicates to use the current working directory.}
  \item{header}{indicator whether the data file
    \code{Z}) includes a header.}
}
\details{
  The function sets up and carries out posterior Markov chain
  Monte Carlo (MCMC) simulation for a hierarchical DP mixture
  model.
  
  See   \code{\link{hdpmn-package}}
  for a statement of the probability model.
}
\value{
  The function returns no value. MCMC simulations
  are saved in files in the designated working directory.

  Use \code{\link{hdpmnPredict}} to plot summaries.
}
\references{
   Mueller, P., Quintana, F. and Rosner, G. (2004).
  ``Hierarchical Meta-Analysis over Related Non-parametric Bayesian
  Models.''
  \emph{Journal of the Royal Statistical Society, Series B}, 66,
  735--749.
}
\author{Peter Mueller \email{pm@wotan.mdacc.tmc.edu}}
\examples{
\dontrun{
###
### hdpm.R - Demo
###

require(hdpmn)

## data files
data.dir <- system.file("data",package="hdpmn")

## data files
Z <- file.path(data.dir,"CALGBz.txt")  
     ## data (first 7 columns) and covariates (last 3 columns)
X <- file.path(data.dir,"CALGBz0.txt")
     ## same as Z, for future patients (for prediction)
S <- file.path(data.dir,"CALGBstudy.txt")
     ## table of patient number and study index
pa.st <- as.matrix(read.table(S,header=T))
study <- pa.st[,2]  # get study index


## run MCMC -- save working files in current working directory
hdpmn(Z=Z,nstudies=3,n.iter=500,study=study,px=3,q=15,cc=15,
      work.dir=work.dir)

## post-process MCMC output for predictive inference
## save posterior predictive simulations in z00 ... z30
z10 <- hdpmnPredict(X=X,px=3,j=1,r=0) # post prediction for study 1
z20 <- hdpmnPredict(X=X,px=3,j=2,r=0) # .. study 2
z30 <- hdpmnPredict(X=X,px=3,j=3,r=0) # .. population at large (= study 3)

z11 <- hdpmnPredict(X=X,px=3,j=1,r=1) # idiosyncratic measures study 1
z21 <- hdpmnPredict(X=X,px=3,j=2,r=1) # .. study 2
z00 <- hdpmnPredict(X=X,px=3,j=0,r=0) # common measure

## covariates (and dummy random effects) of future patients
X <- as.matrix(read.table(X,header=T))
colnames(z00) <- c("PATIENT",colnames(X))

## plot estimated density for future patients in study 1, 2 and
## in population at large
idx <- which(z10[,1]==1)   ## PATIENT 1
options(digits=2)

par(mfrow=c(2,1))          

## plot prediction fo study 1,2,population
plot  (density(z10[idx,8]),
       ylim=c(0,1.5),xlim=c(-0.5,2.5),
       xlab="SLOPE OF RECOVERY",bty="l",main="FUTURE PAT 1")
lines (density(z20[idx,8]),type="l",col=2)
lines (density(z30[idx,8]),type="l",col=3)
legend(-0.5,1.5,col=1:3,legend=c("STUDY 1","STUDY 2","POPULATION"),
       lty=c(1,1,1),bty="n")

## common and idiosyncratic measures
plot (density(z00[idx,8]),type="l",col=4,lty=1,
       ylim=c(0,1.5),xlim=c(-0.5,2.5),
       xlab="SLOPE OF RECOVERY",bty="l",main="COMMON & IDIOSYNC PARTS")
lines (density(z11[idx,8]),type="l",col=1,lty=2)
lines (density(z21[idx,8]),type="l",col=2,lty=2)
legend(1.5,1.5,col=c(1,2,4),lty=c(2,2,1),
       legend=c("STUDY 1 (idiosyn.)",
                "STUDY 2 (idiosyn.)",
                "COMMON"),bty="n")

## plot estimated density for future patients in study 1, 2 and
## in population at large
idx <- which(z10[,1]==2)   ## PATIENT 2
options(digits=2)

par(mfrow=c(2,1))

plot  (density(z10[idx,8]),
       ylim=c(0,1.5),xlim=c(-0.5,2.5),
       xlab="SLOPE OF RECOVERY",bty="l",main="FUTURE PAT 2")
lines (density(z20[idx,8]),type="l",col=2)
lines (density(z30[idx,8]),type="l",col=3)
legend(-0.5,1.5,col=1:3,legend=c("STUDY 1","STUDY 2","POPULATION"),
       lty=c(1,1,1),bty="n")

plot (density(z00[idx,8]),type="l",col=4,lty=1,
       ylim=c(0,1.5),xlim=c(-0.5,2.5),
       xlab="SLOPE OF RECOVERY",bty="l",main="COMMON & IDIOSYNC PARTS")
lines (density(z11[idx,8]),type="l",col=1,lty=2)
lines (density(z21[idx,8]),type="l",col=2,lty=2)
legend(1.5,1.5,col=c(1,2,4),lty=c(2,2,1),
       legend=c("STUDY 1 (idiosyn.)",
                "STUDY 2 (idiosyn.)",
                "COMMON"),bty="n")

## plot nadir count by covariate, for population 
z2 <- z30[,3]; ctx <- z30[,9]; gm <- z30[,10]; amf <- z30[,11]
## fix covariates gm (GM-CSF) and amf (aminofostine)
idx <- which( (gm==-1.78) & (amf== -0.36) )
boxplot(split(z2,ctx),
        xlab="CYCLOPHOSPHAMIDE",bty="n",ylab="NADIR COUNT")
}
}
